import pygame 
from player import Player, ChargeBeam, Explosion, loadImage
import random

def getBackgroundImages():
	images = []
	# gets the background images and stores them in a 2d list
	# 0 - refers to ground objects
	# 1 - refers to windows
	# 2 - refers to doors
	# 3 - refers to billboards
	# 4 - refers to sides
	# 5 - refers to top
	
	# Load ground objects
	im = getImageSet("ground")
	images.append(im)
	im = getImageSet("window")
	images.append(im)
	im = getImageSet("door")
	images.append(im)
	im = getImageSet("billboard")
	images.append(im)
	im = getImageSet("side")
	images.append(im)
	im = getImageSet("top")
	images.append(im)
	return images
def getImageSet(name):
	imageSet = []
	i = 0
	while(True):
		i += 1
		path = "data//images//enviro//" + name + str(i) + ".png"
		try:			
			imageSet.append(loadImage(path))
		except:
			return imageSet
			
class Background(object):
	def __init__(self,screen):
		self.screen = screen
		self.frame = 0
		backcolour = (200,0,0)
		self.xpos = 0
		self.player = Player(screen)
		self.images = getBackgroundImages()
		self.backImage = loadImage("data//images//background.png")
		self.frontOne = FrontLayer(self.screen, 0,self.images)
		self.backOne = Layer(self.screen,0,3)		
		self.backTwo = Layer(self.screen,720,3)
		self.frontTwo = FrontLayer(self.screen, 720,self.images)
		self.middleOne = Layer(self.screen,0,2)
		self.middleTwo = Layer(self.screen,720,2)
	def updateLayers(self):
		#print "whooblah"
		if not self.backOne.onScreen(self):
			self.backOne = self.backTwo
			self.backTwo = Layer(self.screen, self.backOne.xpos + 720,3)
		if not self.frontOne.onScreen(self):
			self.frontOne = self.frontTwo
			self.frontTwo = FrontLayer(self.screen, self.frontOne.xpos + 720,self.images)
		if not self.middleOne.onScreen(self):
			self.middleOne = self.middleTwo
			self.middleTwo = Layer(self.screen, self.middleOne.xpos + 720,2)
	def update(self):
		self.updateLayers()
		if isinstance(self.player.state,ChargeBeam) or isinstance(self.player.state,Explosion):
			self.frame += 1
			return
		#self.xpos += 1
		#print self.xpos
		self.frame += 1
	def draw(self,player):
		if not (isinstance(self.player.state,ChargeBeam) or isinstance(self.player.state,Explosion)):
			if player.pos[0]-320 > self.xpos:
				self.xpos += (player.pos[0]-320-self.xpos)/8+1		
		r =self.backImage.get_rect()
		self.screen.blit(self.backImage,r)
		pygame.draw.rect(self.screen,(0,0,0),(0,300,640,60),0)
		self.backOne.draw(self)
		self.backTwo.draw(self)
		self.middleOne.draw(self)
		self.middleTwo.draw(self)
		self.frontOne.draw(self)
		self.frontTwo.draw(self)
		self.player = player
	

		
class Layer(object):
	def __init__(self,screen,xpos,depth):
		self.screen = screen
		self.xpos = xpos
		self.depth = depth
		self.rects = []
		self.randomize()
	def randomize(self):
		for i in range(10):
			r = pygame.Rect(0,0,int(random.random()*30)*3+30,(int(random.random()*50)+5)*(6-int(self.depth*1.5)))
			r.bottom = 300
			r.left = random.random()*(720-r.width)
			self.rects.append(r)
		
	def onScreen(self,background):		
		if self.xpos + 720 < background.xpos/self.depth:
			return False
		return True
	def draw(self,background):
		for r in self.rects:
			drawrect = r.copy()
			drawrect.centerx += self.xpos - background.xpos/self.depth
			d = (self.depth**0.5)*(3**0.5)
			pygame.draw.rect(self.screen,(60+int(25*d),60+int(30*d),60+int(40*d)),drawrect,0)

class FrontLayer(Layer):
	def __init__(self,screen,xpos,images):
		self.images = images
		self.image = pygame.surface.Surface((720,360))
		Layer.__init__(self,screen,xpos,1)
		
	def randomize(self):
		self.image.fill((255,0,255))		
		left = 0
		right = 720
		curLeft = True
		while (right - left) > 62:
			# Get building size and side to put something on
			leftSide = False
			hasSide = False
			maxSize = int((right - left - 17)/30)
			if maxSize > 2:
				if maxSize > 6:
					maxSize = 6				
				xSize = int((random.random()**1.8)*(maxSize-2))+3
				ySize = int(random.random()*7)+2
				if random.random() > 0.5:
					leftSide = True
					hasSide = True
				else:
					leftSide = False
					hasSide = True
			else:
				
				maxSize = int((right - left)/30)
				if maxSize == 3:
					xSize = 3
					ySize = int(random.random()*6)+2
				else:
					maxSize = int((right - left - 17)/30)
					xSize = 2
					ySize = int(random.random()*4)+2
					if maxSize == 2:
						if random.random() > 0.5:
							leftSide = True
							hasSide = True
						else:
							leftSide = False
							hasSide = True					
			# Draw base outline for building

			r = pygame.Rect(0,0,xSize*30,ySize*35)
			r.bottom = 300
			a = int(random.random()*xSize)+1
			if hasSide:
				if curLeft:
					if leftSide:						
						r.left = 13+a+left
					else:
						r.left = a+left
					left += 13+a+xSize*30
				else:
					if leftSide:
						r.right = right - a
					else:
						r.right = right - 13 - a
					right -= (13+a+xSize*30)
						
			else:
				if curLeft:
					r.left = left - a
					left += a+xSize*30
				else:
					r.right = right - a
					right -= (a+xSize*30)
			
			c1 = int(132+random.random()*6)
			c2 = int(157+random.random()*6)
			pygame.draw.rect(self.image,(c2+int(random.random()*6)-2,c2,c2),r,0)
			pygame.draw.rect(self.image,(c1+int(random.random()*6)-2,c1,c1),r,1)
			usedPos = []
			# Create Door
			doorPos = (int(random.random()*xSize),0)
			usedPos.append(doorPos)
			door = self.images[2][int(random.random()*len(self.images[2]))]
			dr = door.get_rect()
			dr.left = r.left+doorPos[0] * 30
			dr.bottom = 300 - doorPos[1]*35
			self.image.blit(door,dr)
			
			# Create windows
			br = int(random.random()*8) - 3
			for w in range((xSize*ySize)/3):
				wPos = (int(random.random()*xSize),int(random.random()*(ySize-1))+1)
				if wPos not in usedPos:
					usedPos.append(wPos)
					window = self.images[1][int(random.random()*len(self.images[1]))]
					wr = window.get_rect()
					wr.left = r.left+wPos[0] * 30
					wr.bottom = 300 - wPos[1]*35 - br
					self.image.blit(window,wr)
			# Add sidepiece
			if hasSide:
				side = self.images[4][int(random.random()*len(self.images[4]))]
				sr = side.get_rect()
				hp = int(random.random()*(r.height - sr.height - 2)) + 1
				sr.top = r.top + hp
				if leftSide:
					side = pygame.transform.flip(side,True,False)
					sr.right = r.left
				else:
					sr.left = r.right
				self.image.blit(side,sr)
			# Add top
			side = self.images[5][int(random.random()*len(self.images[5]))]
			sr = side.get_rect()
			wp = int(random.random()*(r.width - sr.width - 2)) + 1
			sr.left = r.left + wp
			sr.bottom = r.top
			self.image.blit(side,sr)
			# billboard
			if random.random() > 0.7:
				if ySize > 2:
					wPos = (int(random.random()*2)*(xSize-1),ySize-int(random.random()*2)-1)				
					wPos2 = (int(random.random()*2)*(xSize-1),ySize-int(random.random()*2)-1)				
					if wPos2 not in usedPos:
						wPos = wPos2
					if wPos not in usedPos:
						usedPos.append(wPos)
						window = self.images[3][int(random.random()*len(self.images[3]))]
						wr = window.get_rect()
						wr.centerx = r.left+wPos[0] * 30+15+int(random.random()*5)-2
						wr.centery = 300 - wPos[1]*35 - br - 18
						self.image.blit(window,wr)
					
			if curLeft:
				left += int(random.random()*25+10)
			else:
				right -= int(random.random()*25+10)
			curLeft = not curLeft			
		r = int(random.random()*5)+2
		for i in range(r):
			front = self.images[0][int(random.random()*len(self.images[0]))]
			if random.random() > 0.5:
				front = pygame.transform.flip(front,True,False)
			fr = front.get_rect()
			fr.bottom = 300
			fr.centerx = int(random.random()*((720/r)-fr.width))+(fr.width/2)+(720/r)*i
			self.image.blit(front,fr)
			
			
					
		self.image.set_colorkey((255,0,255))
	def draw(self,background):		
		drawrect = self.image.get_rect()		
		drawrect.left += self.xpos - background.xpos/self.depth
		self.screen.blit(self.image,drawrect)